"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateServiceWorker = void 0;
const chalk_1 = require("chalk");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const workbox_build_1 = require("workbox-build");
const utils_1 = require("./utils");
const imageFilter = (outDir, maxsize = 1024) => (manifestEntries) => {
    const warnings = [];
    const manifest = [];
    const imageExtensions = [".png", ".jpg", ".jpeg", "webp", "bmp", "gif"];
    for (const entry of manifestEntries)
        if (imageExtensions.some((ext) => entry.url.endsWith(ext))) {
            const stats = (0, fs_extra_1.statSync)((0, path_1.resolve)(outDir, entry.url));
            if (stats.size > maxsize * 1024)
                warnings.push(`Skipped ${entry.url}, as its ${Math.ceil(stats.size / 1024)} KB.\n`);
            else
                manifest.push(entry);
        }
        else
            manifest.push(entry);
    return { warnings, manifest };
};
const generateServiceWorker = async ({ outDir, siteConfig }, options) => {
    var _a;
    utils_1.logger.load("Generating service worker");
    const { title, locales = {} } = siteConfig;
    const swDest = (0, path_1.resolve)(outDir, "./service-worker.js");
    const globPatterns = ["**/*.{js,css,svg}", "**/*.{woff,woff2,eot,ttf,otf}"];
    if (options.cacheHTML)
        globPatterns.push("**/*.html");
    else
        globPatterns.push("./index.html", "./404.html");
    if (options.cachePic)
        globPatterns.push("**/*.{png,jpg,jpeg,bmp,gif,webp}");
    await (0, workbox_build_1.generateSW)({
        swDest,
        globDirectory: outDir,
        cacheId: title || ((_a = locales["/"]) === null || _a === void 0 ? void 0 : _a.title) || "hope",
        globPatterns,
        cleanupOutdatedCaches: true,
        clientsClaim: true,
        maximumFileSizeToCacheInBytes: (options.maxSize || 2048) * 1024,
        manifestTransforms: [imageFilter(outDir, options.maxPicSize)],
        ...(options.generateSWConfig || {}),
    }).then(({ count, size, warnings }) => {
        utils_1.logger.succeed();
        utils_1.logger.info(`Precache ${(0, chalk_1.cyan)(`${count} files`)}, totaling ${(0, chalk_1.cyan)(`${(size / 1024 / 1024).toFixed(2)} Mb.`)}.`);
        if (warnings.length)
            utils_1.logger.warn(`${warnings.map((warning) => `  · ${warning}`).join("\n")}`);
        if (size > 104857600)
            utils_1.logger.error(`Cache Size is larger than 100MB, so that it can not be registerd on all browsers.\n${(0, chalk_1.blue)("Please consider disable `cacheHTML` and `cachePic`, or set `maxSize` and `maxPicSize` option.\n")}`);
        else if (size > 52428800)
            utils_1.logger.warn(`\nCache Size is larger than 50MB, which will not be registerd on Safari.\n${(0, chalk_1.blue)("Please consider disable `cacheHTML` and `cachePic`, or set `maxSize` and `maxPicSize` option.\n")}`);
    });
};
exports.generateServiceWorker = generateServiceWorker;
//# sourceMappingURL=generateServiceWorker.js.map